@@ -1,15 +1,11 @@
 use crate::parsing::ast;
+use crate::utils::{get_workspace_dir, write_file};
 use anyhow::{Context, Result};
 use glob::glob;
 use quote::quote;
-use serde::Deserialize;
-use std::{
-    collections::HashSet,
-    env,
-    fs::{create_dir_all, File},
-    io::Write,
-    path::{Path, PathBuf},
-};
+use std::collections::BTreeSet;
+use std::env;
+use std::path::{Path, PathBuf};
 
 #[allow(clippy::needless_doctest_main)]
 /// CSS Modules compiler.
@@ -29,7 +25,9 @@ use std::{
 /// ```
 #[derive(Debug, Default)]
 pub struct Compiler {
-    input_modules: HashSet<PathBuf>,
+    // use sorted set instead of hash set so output bundle is rendered deterministically with the
+    // same content across recompilations
+    input_modules: BTreeSet<PathBuf>,
 }
 
 impl Compiler {
@@ -43,10 +41,7 @@ impl Compiler {
     ///
     /// * `path`: File path, which may be absolute or relative to package root directory.
     pub fn add_module(&mut self, path: &str) -> Result<&mut Self> {
-        let path = PathBuf::from(path);
-
-        self.add_module_buf(path)?;
-
+        self.add_module_buf(PathBuf::from(path))?;
         Ok(self)
     }
 
@@ -59,7 +54,6 @@ impl Compiler {
         for entry in glob(pattern).context("Failed to read glob pattern")? {
             self.add_module_buf(entry?)?;
         }
-
         Ok(self)
     }
 
@@ -77,8 +71,8 @@ impl Compiler {
 
     /// Parses and transforms input CSS modules.
     ///
-    /// Generates CSS bundle file ready to include in HTML page, and name mappings rust code ready
-    /// to include into rust project with `css_mod::init!()`.
+    /// Generates CSS bundle file ready to be included on a HTML page, and rust code file with name
+    /// mappings ready to be included into rust project with [`css_mod::init!`](crate::init).
     ///
     /// Arguments:
     ///
@@ -101,7 +95,7 @@ impl Compiler {
         let mut css_bundle_content = String::new();
         let mut mappings_code_content = String::new();
 
-        let workspace_dir = Self::get_workspace_dir()?;
+        let workspace_dir = get_workspace_dir()?;
         log::debug!("workspace dir: {:?}", workspace_dir);
 
         let is_windows_host = cfg!(windows);
@@ -163,7 +157,7 @@ impl Compiler {
         }
 
         log::debug!("output css bundle: {:?}", css_bundle_path);
-        Self::write(&css_bundle_path, css_bundle_content)?;
+        write_file(&css_bundle_path, css_bundle_content)?;
 
         // output mappings code
         let out_dir = env::var("OUT_DIR").context(
@@ -174,41 +168,8 @@ impl Compiler {
 
         let mappings_code_file_path = &out_dir.join(crate::MAPPINGS_FILE_NAME!());
         log::debug!("output mappings code: {:?}", mappings_code_file_path);
-        Self::write(mappings_code_file_path, mappings_code_content)?;
+        write_file(mappings_code_file_path, mappings_code_content)?;
 
         Ok(())
     }
-
-    fn write(file_path: &Path, content: String) -> Result<()> {
-        let dir_path = file_path
-            .parent()
-            .context("Failed to get parent directory")?;
-        create_dir_all(&dir_path)?;
-
-        let mut file = File::create(&file_path)
-            .with_context(|| format!("Failed to create file: {:?}", file_path))?;
-        file.write_all(content.as_bytes())?;
-
-        Ok(())
-    }
-
-    /// Gets path to workspace root directory of currently built package, or package root directory
-    /// if it is not part of workspace.
-    fn get_workspace_dir() -> Result<PathBuf> {
-        // this is ugly but the only way to get workspace directory path right now
-        // TODO: replace with environment variable when cargo supports it
-        // https://github.com/rust-lang/cargo/issues/3946
-        #[derive(Deserialize)]
-        struct Manifest {
-            workspace_root: String,
-        }
-        let package_dir = env::var("CARGO_MANIFEST_DIR").context("CARGO_MANIFEST_DIR")?;
-        let output = std::process::Command::new(env!("CARGO"))
-            .arg("metadata")
-            .arg("--format-version=1")
-            .current_dir(&package_dir)
-            .output()?;
-        let manifest: Manifest = serde_json::from_slice(&output.stdout)?;
-        Ok(PathBuf::from(manifest.workspace_root))
-    }
 }
